{% load apis_helpers %}
<button type='button' class='btn btn-primary btn-sm reledit' onclick=GetFormAjax("{{ record|content_type}}Form","{% if record.relation_pk %}{{record.relation_pk}}{% else %}{{ record.pk}}{% endif %}")>Edit</button>
