# Generated by Django 2.1.2 on 2018-11-21 15:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('apis_relations', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='placeplace',
            name='related_placeA',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_placeA', to='apis_entities.Place'),
        ),
        migrations.AlterField(
            model_name='placeplace',
            name='related_placeB',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_placeB', to='apis_entities.Place'),
        ),
    ]
