{% load apis_helpers %}
<button id="tempEntity_{% if record.relation_pk %}{{ record.relation_pk }}{% else %}{{ record.pk }}{% endif %}" type="button" class="btn btn-default reldelete" aria-label="Left Align" onclick=DeleteTempEntity('{% if record.relation_pk %}{{record.relation_pk}}{% else %}{{ record.pk }}{% endif %}',"{{ record|content_type|lower}}")><span class="glyphicon glyphicon-remove" aria-hidden="true"></span></button>
